/* animobject.js */


Janice._AnimObject = {};
Janice._AnimObject.method = {};
Janice._AnimObject.staticm = {};


Janice._AnimObject.staticm.save = function(object, extraData) {
    var saveStr = '{';

    saveStr += '"t":' + '"' + object.type + '"';
    saveStr += ',';

    saveStr += '"p":[';
    var firstProperty = true;
    for (var p in object._properties) {
        if (firstProperty) { firstProperty = false; } else { saveStr += ','; }
        var property = object._properties[p];
        saveStr += '{';
        saveStr += '"n":' + '"' + p + '"';
        saveStr += ',';
        saveStr += '"t":' + '"' + property.type + '"';
        saveStr += ',';
        var value = Janice.PropertyTypes[property.type].toString(property.value);
        value = typeof value === 'string' ? '"' + value.replace('"', '\\"') + '"' : value;
        saveStr += '"v":' + value;
        if (property.modifications.length > 0) {
            saveStr += ',';
            saveStr += '"m":[';
            var firstMod = true;
            for (var i = 0; i < property.modifications.length; ++i) {
                if (firstMod) { firstMod = false; } else { saveStr += ','; }
                var mod = property.modifications[i];
                saveStr += '{';
                saveStr += '"t":';
                saveStr += '{';
                saveStr += '"s":' + mod.transition.startTime;
                saveStr += ',';
                saveStr += '"e":' + mod.transition.endTime;
                saveStr += ',';
                saveStr += '"f":' + '"' + mod.transition.funcName + '"';
                saveStr += '}';
                saveStr += ',';
                var endValue = Janice.PropertyTypes[property.type].toString(mod.endValue);
                endValue = typeof endValue === 'string' ? '"' + endValue.replace('"', '\\"') + '"' : endValue;
                saveStr += '"v":' + endValue;
                saveStr += '}';
            }
            saveStr += ']';
        }
        saveStr += '}';
    }
    saveStr += ']';

    if (extraData) {
        saveStr += ',' + extraData;
    }

    return saveStr + '}';
};


Janice._AnimObject.staticm.load = function(data, overrideBlueprint) {
    var object = Janice.AnimObject(overrideBlueprint || data.t);

    var properties = {};
    for (var i = 0; i < data.p.length; ++i) {
        var propertyData = data.p[i];

        properties[propertyData.n] = {
            type: propertyData.t,
            value: Janice.PropertyTypes[propertyData.t].fromString(propertyData.v)
        };
    }
    object._addProperties(properties);

    for (var i = 0; i < data.p.length; ++i) {
        var propertyData = data.p[i];

        if ('m' in propertyData) {
            for (var j = 0; j < propertyData.m.length; ++j) {
                var mod = propertyData.m[j];
                object.addModification(propertyData.n,
                    Janice.Transition(mod.t.s, mod.t.e, mod.t.f, false),
                    Janice.PropertyTypes[propertyData.t].fromString(mod.v));
            }
        }
    }

    return object;
};


Janice._AnimObject.staticm.defaultProperties = {
    show: { type: 'bool', value: true},
    x: { type: 'number', value: 0 },
    y: { type: 'number', value: 0 },
    scale: { type: 'number', value: 1 },
    rotation: { type: 'number', value: 0 }
};


Janice._AnimObject.method.addProperties = function(properties) {
    for (var p in properties) {
        this._properties[p] = {
            type: properties[p].type,
            value: properties[p].value,
            modifications: []
        };
    }
};


Janice._AnimObject.staticm.getPropertyValueAt = function(property, t) {
    var value = property.value;
    for (var i = 0; i < property.modifications.length; ++i) {
        var modification = property.modifications[i];
        if (i == property.modifications.length - 1 || t <= modification.transition.endTime) {
            var trVal = modification.transition.boundedValueAt(t);
            return Janice.PropertyTypes[property.type].interpolate(value, modification.endValue, trVal);
        }
        else {
            value = modification.endValue;
        }
    }

    return value;
};


Janice._AnimObject.method.setPropertyValue = function(propertyKey, value) {
    this._properties[propertyKey].value = value;
};


Janice._AnimObject.method.addModification = function(propertyKey, newTransition, endValue) {
    // Check where to insert and whether non-overlapping:
    var insertIndex = -1;
    for (var i = 0; i < this._properties[propertyKey].modifications.length; ++i) {
        var transition = this._properties[propertyKey].modifications[i].transition;
        if (newTransition.endTime <= transition.startTime) {
            insertIndex = i;
            break;
        }
        else if (newTransition.startTime >= transition.endTime) {
            insertIndex = i + 1;
            continue;
        }
        else {
            throw 'OverlappingTransitionsError';
        }
    }

    var mod = {
        transition: newTransition,
        endValue: endValue
    };

    if (insertIndex < this._properties[propertyKey].modifications.length) {
        this._properties[propertyKey].modifications.splice(insertIndex, 0, mod);
    }
    else {
        this._properties[propertyKey].modifications.push(mod);
    }
};


Janice._AnimObject.method.isShownAt = function(t) {
    return Janice._AnimObject.staticm.getPropertyValueAt(this._properties.show, t);
};


Janice._AnimObject.method.draw = function(context, t) {
    if (this.isShownAt(t)) {
        // Compute property values:
        var propertyValues = {};
        for (var p in this._properties) {
            propertyValues[p] = Janice._AnimObject.staticm.getPropertyValueAt(this._properties[p], t);
        }

        // Adjust transformation matrix and draw:
        context.save();
        context.translate(propertyValues.x, propertyValues.y);
        context.scale(propertyValues.scale, propertyValues.scale);
        context.rotate(propertyValues.rotation * Math.PI / 180.0);
        this._draw(context, propertyValues, t);
        context.restore();
    }
};


Janice.AnimObject = function(blueprint, overrideProperties) {
    if (typeof(blueprint) == 'string') {
        blueprint = Janice.BlueprintManager.retrieveBlueprint(blueprint);
        if (!blueprint) {
            throw 'UndefinedBlueprint';
        }
    }

    var object = {};
    object.type = blueprint.type;

    object._draw = blueprint.draw;
    object.draw = Janice._AnimObject.method.draw;

    object.isShownAt = Janice._AnimObject.method.isShownAt;

    object._addProperties = Janice._AnimObject.method.addProperties;
    object.setPropertyValue = Janice._AnimObject.method.setPropertyValue;
    object.addModification = Janice._AnimObject.method.addModification;

    object._properties = {};
    object._addProperties(Janice._AnimObject.staticm.defaultProperties);
    object._addProperties(blueprint.properties);
    if (overrideProperties) {
        for (var p in overrideProperties) {
            object.setPropertyValue(p, overrideProperties[p]);
        }
    }

    return object;
};

