/* transition.js */


Janice.TransitionFunctions = {
    functions: {},
    register: function(name, func, invFunc) {
        this.functions[name] = func;
        this.functions['inv_' + name] = invFunc || function(t) { return 1 - func(1-t); };
    },
    funcName: function(name, inverse) { return (inverse ? 'inv_' : '') + name; },
    retrieve: function(name, inverse) {
        return this.functions[this.funcName(name, inverse)];
    }
};


// Default transition functions:
Janice.TransitionFunctions.register('linear',
    function(t) { return t; },
    function(t) { return t; }
);

Janice.TransitionFunctions.register('quadratic',
    function(t) { return t*t; },
    function(t) { return (2-t)*t; }
);


Janice._Transition = {};
Janice._Transition.method = {};


Janice._Transition.method.valueAt = function(t) {
    return this._func((t - this.startTime) / this.duration);
};


Janice._Transition.method.valueAtForZeroDuration = function(t) {
    return 0.5;
};


Janice._Transition.method.boundedValueAt = function(t) {
    var ti = (t - this.startTime) / this.duration;
    return ti <= 0 ? 0 : ti >= 1 ? 1 : this._func(ti);
};


Janice._Transition.method.boundedValueAtForZeroDuration = function(t) {
    return t < this.startTime ? 0 : 1;
};


Janice._Transition.method.adjustStartAndEndTime = function(startTime, endTime) {
    this.startTime = startTime === null ? this.startTime : startTime;
    this.endTime = endTime === null ? this.endTime : endTime;
    this.duration = endTime - startTime;

    this.valueAt = this.duration !== 0 ?
        Janice._Transition.method.valueAt :
        Janice._Transition.method.valueAtForZeroDuration;
    this.boundedValueAt = this.duration !== 0 ?
        Janice._Transition.method.boundedValueAt :
        Janice._Transition.method.boundedValueAtForZeroDuration;
};


Janice.Transition = function(startTime, endTime, funcName, inverse)
{
    funcName = funcName || 'linear';

    var transition = {};

    transition.funcName = Janice.TransitionFunctions.funcName(funcName, inverse);
    transition._func = Janice.TransitionFunctions.retrieve(funcName, inverse);

    transition.adjustStartAndEndTime = Janice._Transition.method.adjustStartAndEndTime;
    transition.adjustStartAndEndTime(startTime, endTime);

    return transition;
};


Janice.Switch = function(time) {
    return Janice.Transition(time, time);
};

