
var _JanicePlayer = {};
_JanicePlayer.method = {};


_JanicePlayer.method.draw = function() {
    this._canvas.width = this._canvas.width; // Clear the canvas!
    this._context.save();
    this.animation.draw(this._context, this.time);
    this._context.restore();
};


_JanicePlayer.method.update = function() {
    var time = (new Date()).getTime() - this._playStartTime;
    if (time < this.animation.duration) {
        this.time = time;
        this._draw();
        var self = this;
        this._timeout = setTimeout(function() { self._update(); }, 10);
    }
    else if (this.time !== this.animation.duration) {
        this.time = this.animation.duration;
        this.pause();
        this._draw();
    }
};


_JanicePlayer.method.play = function() {
    if (!this._timeout) {
        this._playStartTime = (new Date()).getTime() - this.time;
        this._update();
    }
};


_JanicePlayer.method.pause = function() {
    if (this._timeout) {
        clearTimeout(this._timeout);
        this._timeout = null;
    }
};


_JanicePlayer.method.jump = function(jumpTime) {
    jumpTime = jumpTime < 0 ? 0 :
        jumpTime > this.animation.duration ? this.animation.duration : parseInt(jumpTime);
    this._playStartTime -= jumpTime - this.time;
    this.time = jumpTime;
    if (!this._timeout) {
        this._draw();
    }
};


_JanicePlayer.method.reset = function() {
    this.pause();
    this.jump(0);
};


_JanicePlayer.method.load = function(data) {
    try {
        this.animation.load(data);
    }
    catch (e) {
        return false; // Failure.
    }

    if (this._autoResize) {
        this._canvas.width = this.animation.width;
        this._canvas.height = this.animation.height;
    }
    else {
        // TODO: Set scale and clipping region.
    }

    this.reset();

    return true; // Success.
};


function JanicePlayer(canvas, autoResize) {
    this._canvas = canvas;
    this._autoResize = autoResize ? true : false;
    this._context = canvas.getContext('2d');

    this.animation = Janice.Animation();

    this.time = 0;

    this._playStartTime = 0;
    this._timeout = null;

    this._draw = _JanicePlayer.method.draw;
    this._update = _JanicePlayer.method.update;

    this.play = _JanicePlayer.method.play;
    this.pause = _JanicePlayer.method.pause;
    this.jump = _JanicePlayer.method.jump;
    this.reset = _JanicePlayer.method.reset;

    this.load = _JanicePlayer.method.load;
}

