/* janice_creator.js */

var JaniceCreator = {};


JaniceCreator.onDomReadyFunctions = [];
JaniceCreator.registerOnDomReadyFunction = function(f) {
    JaniceCreator.onDomReadyFunctions.push(f);
};
JaniceCreator.callOnDomReadyFunctions = function() {
    JaniceCreator.onDomReadyFunctions.each(function(f) { f(); });
};


JaniceCreator.canvas = null;
JaniceCreator.context = null;
JaniceCreator.animation = null;
JaniceCreator.currentTime = 0;
JaniceCreator.containerStack = [];
JaniceCreator.currentAnimObjectId = -1;


JaniceCreator.updateAnimation = function() {
    JaniceCreator.canvas.width = JaniceCreator.canvas.width; // Clear the canvas!
    JaniceCreator.animation.draw(JaniceCreator.context, JaniceCreator.currentTime);

    JaniceCreator.updateObjectDataView();
};


JaniceCreator.newAnimation = function() {
    JaniceCreator.animation = Janice.Animation();
    JaniceCreator.loadAnimData(true);
};


JaniceCreator.loadAnimData = function(isNewAnimation) {
    if (isNewAnimation !== true) {
        // Load the data in the animation instance:
        var data = $('animData').value;
        JaniceCreator.animation.load(data);
    }

    // Set the canvas size:
    JaniceCreator.canvas.width = JaniceCreator.animation.width;
    JaniceCreator.canvas.height = JaniceCreator.animation.height;

    // Set animation length:
    $('animLength').value = JaniceCreator.animation.duration;

    // Reset current time:
    JaniceCreator.currentTime = 0;
    $('currentTime').value = JaniceCreator.currentTime;

    // Set the top-level container:
    JaniceCreator.containerStack = [JaniceCreator.animation.container];
    JaniceCreator.updateContainerStackView();

    // Draw the animation:
    JaniceCreator.updateAnimation();
};


JaniceCreator.saveAnimData = function() {
    // Load the data in the animation instance:
    $('animData').value = JaniceCreator.animation.save();
};


JaniceCreator.changeCurrentTime = function() {
    var currentTime = parseInt($('currentTime').value);
    JaniceCreator.currentTime = currentTime;
    JaniceCreator.updateAnimation();
};


JaniceCreator.changeAnimLength = function() {
    JaniceCreator.animation.duration = parseInt($('animLength').value);
};


JaniceCreator.updateContainerStackView = function() {
    // Update the container stack view:
    var containerStackElem = $('containerStack');
    containerStackElem.childElements().each(function(elem) { elem.remove(); });
    for (var i = 0; i < JaniceCreator.containerStack.length; ++i) {
        var object = JaniceCreator.containerStack[i];
        var objectElem = new Element('li', { 'class': 'containerItem', 'id': 'containerItem_' + i }).update(object.type);
        containerStackElem.insert(objectElem);
    }

    // Update the objects view:
    JaniceCreator.updateAnimObjectsView();
};


JaniceCreator.updateAnimObjectsView = function() {
    var animObjectsElem = $('animObjects');
    animObjectsElem.childElements().each(function(elem) { elem.remove(); });

    var currentContainer = JaniceCreator.containerStack[JaniceCreator.containerStack.length - 1];
    var animObjects = currentContainer._objects;

    for (var objectId in animObjects) {
        var object = animObjects[objectId];
        var objectElem = new Element('li', { 'class': 'animObjectItem', 'id': 'animObjectItem_' + objectId }).update(object.type);
        if (object.type === 'container') {
            objectElem.addClassName('container');
        }
        animObjectsElem.insert(objectElem);
    }

    JaniceCreator.setCurrentAnimObject(0);
};


JaniceCreator.onClickAnimObjects = function(event) {
    var target = event.target;
    if (target.hasClassName('animObjectItem')) {
        var objectId = parseInt(target.id.replace('animObjectItem_', ''));
        if (target.hasClassName('container')) {
            JaniceCreator.containerStack.push(JaniceCreator.containerStack[JaniceCreator.containerStack.length - 1]._objects[objectId]);
            JaniceCreator.updateContainerStackView();
        }
        else {
            JaniceCreator.setCurrentAnimObject(objectId);
        }
    }
};


JaniceCreator.onClickContainerStack = function(event) {
    var target = event.target;
    if (target.hasClassName('containerItem')) {
        var clickedIndex = parseInt(target.id.replace('containerItem_', ''));
        while (JaniceCreator.containerStack.length > clickedIndex + 1) {
            JaniceCreator.containerStack.pop();
        }
        JaniceCreator.updateContainerStackView();
    }
};


JaniceCreator.setCurrentAnimObject = function(objectId) {
    // Remove 'current' class from all object items:
    $('animObjects').select('.current').each(function(elem) { elem.removeClassName('current'); });
    $('containerStack').select('.current').each(function(elem) { elem.removeClassName('current'); });

    if (objectId === 0) {
        $('containerItem_' + (JaniceCreator.containerStack.length - 1)).addClassName('current');
    }
    else if (objectId !== -1) {
        $('animObjectItem_' + objectId).addClassName('current');
    }

    JaniceCreator.currentAnimObjectId = objectId;

    JaniceCreator.updateObjectPropertiesView();
};


JaniceCreator.updateObjectPropertiesView = function() {
    var objectPropertiesElem = $('objectProperties');
    objectPropertiesElem.update('');

    var objectId = JaniceCreator.currentAnimObjectId;

    var animObject = null;
    if (objectId === 0) {
        animObject = JaniceCreator.containerStack[JaniceCreator.containerStack.length - 1];
    }
    else if (objectId !== -1) {
        var currentContainer = JaniceCreator.containerStack[JaniceCreator.containerStack.length - 1];
        animObject = currentContainer._objects[objectId];
    }

    if (animObject) {
        for (var propertyName in animObject._properties) {
            var property = animObject._properties[propertyName];
            var propertyElem = JaniceCreator.createPropertyElem(animObject, propertyName, property);
            objectPropertiesElem.insert(propertyElem);
        }
    }
};


JaniceCreator.createPropertyElem = function(animObject, propertyName, property) {
    var currentValue = Janice._AnimObject.staticm.getPropertyValueAt(property, JaniceCreator.currentTime);
    var currentValueString = Janice.PropertyTypes[property.type].toString(currentValue);

    var propertyElem = new Element('div', { 'class': 'property', 'id': 'property_' + propertyName });

    var toggleModificationsElem = new Element('div', { 'class': 'toggleModifications' }).update('+');
    propertyElem.insert(toggleModificationsElem);

    var nameElem = new Element('div', { 'class': 'name' }).update(propertyName);
    propertyElem.insert(nameElem);

    var currentValueElem = new Element('span', { 'class': 'currentValue' }).update(currentValueString);
    propertyElem.insert(currentValueElem);

    var valueString = Janice.PropertyTypes[property.type].toString(property.value);
    var valueElem = new Element('span', { 'class': 'value' }).update('Start value: ');
    var startValueInputElem = new Element('input', {
        'type': 'text',
        'value': valueString,
        'class': 'propertyStartValue',
        'id': 'propertyStartValue_' + propertyName,
    });
    valueElem.insert(startValueInputElem);
    startValueInputElem.observe('change', JaniceCreator.onPropertyStartValueChange);
    propertyElem.insert(valueElem);

    var modificationsElem = new Element('div', { 'class': 'modifications', 'id': 'modifications_' + propertyName });

    var addTransitionElem = new Element('input', { 'type': 'button', 'value': 'Add transition...' });
    addTransitionElem.observe('click', function() {
        var startTime = 0;
        if (property.modifications.length > 0) {
            startTime = property.modifications[property.modifications.length - 1].transition.endTime;
        }
        animObject.addModification(propertyName, Janice.Transition(startTime, startTime, 'linear', false), property.value);
        JaniceCreator.updatePropertyModifications(modificationsElem, property);
        modificationsElem.show();
    });
    propertyElem.insert(addTransitionElem);

    JaniceCreator.updatePropertyModifications(modificationsElem, property);
    modificationsElem.hide();
    propertyElem.insert(modificationsElem);

    // Attach event observers:
    toggleModificationsElem.observe('click', function(event) { modificationsElem.toggle(); });

    return propertyElem;
};


JaniceCreator.updatePropertyModifications = function(modificationsElem, property) {
    modificationsElem.update('');

    for (var i = 0; i < property.modifications.length; ++i) {
        var modification = property.modifications[i];
        var modificationElem = new Element('div', { 'class': 'transition' });
        modificationElem.insert(new Element('span').update('Transition'));

        modificationElem.insert(new Element('span').update('Start time'));

        var startTimeInputElem = new Element('input', { 'type': 'text', 'value': modification.transition.startTime, 'size': 6 });
        startTimeInputElem.observe('change', function() {
            modification.transition.adjustStartAndEndTime(parseInt(startTimeInputElem.value), null);
            JaniceCreator.updateAnimation();
        });
        modificationElem.insert(startTimeInputElem);

        modificationElem.insert(new Element('span').update('End time'));

        var endTimeInputElem = new Element('input', { 'type': 'text', 'value': modification.transition.endTime, 'size': 6 });
        endTimeInputElem.observe('change', function() {
            modification.transition.adjustStartAndEndTime(null, parseInt(endTimeInputElem.value));
            JaniceCreator.updateAnimation();
        });
        modificationElem.insert(endTimeInputElem);
        modificationElem.insert(new Element('span').update('End value'));

        var endValueString = Janice.PropertyTypes[property.type].toString(modification.endValue);
        var endValueInputElem = new Element('input', { 'type': 'text', 'value': endValueString });
        endValueInputElem.observe('change', function() {
            modification.endValue = Janice.PropertyTypes[property.type].fromString(endValueInputElem.value);
            JaniceCreator.updateAnimation();
        });
        modificationElem.insert(endValueInputElem);

        modificationsElem.insert(modificationElem);
    }
};


JaniceCreator.getPropertyByName = function(propertyName) {
    var objectId = JaniceCreator.currentAnimObjectId;
    var currentContainer = JaniceCreator.containerStack[JaniceCreator.containerStack.length - 1];
    var animObject = null;
    if (objectId === 0) {
        animObject = currentContainer;
    }
    else {
        animObject = currentContainer._objects[objectId];
    }
    var property = animObject._properties[propertyName];
    return property;
};


JaniceCreator.onPropertyStartValueChange = function(event) {
    var target = event.target;
    if (target.hasClassName('propertyStartValue')) {
        var propertyName = target.id.replace('propertyStartValue_', '');
        var property = JaniceCreator.getPropertyByName(propertyName);
        property.value = Janice.PropertyTypes[property.type].fromString(target.value);
        JaniceCreator.updateAnimation();
    }
};


JaniceCreator.updateObjectDataView = function() {
    // Update property current-value:
    $$('.property').each(function(elem) {
        var propertyName = elem.id.replace('property_', '');
        var property = JaniceCreator.getPropertyByName(propertyName);

        var currentValue = Janice._AnimObject.staticm.getPropertyValueAt(property, JaniceCreator.currentTime);
        var currentValueString = Janice.PropertyTypes[property.type].toString(currentValue);
        elem.down('.currentValue').update(currentValueString);
    });
};


JaniceCreator.onClickCreateAnimObject = function() {
    var objectTypesElem = $('objectTypes');
    var objectType = objectTypesElem.options[objectTypesElem.selectedIndex].value;

    var animObject = null;
    if (objectType === 'container') {
        animObject = Janice.Container();
    }
    else {
        animObject = Janice.AnimObject(objectType);
    }
    var currentContainer = JaniceCreator.containerStack[JaniceCreator.containerStack.length - 1];
    currentContainer.addObject(animObject);

    JaniceCreator.updateAnimObjectsView();
    JaniceCreator.updateAnimation();
};


JaniceCreator.registerOnDomReadyFunction(function() {
    var canvas = $('cnvs');
    JaniceCreator.canvas = canvas;
    JaniceCreator.context = canvas.getContext('2d');
    $('newAnim').observe('click', JaniceCreator.newAnimation);
    $('loadAnimData').observe('click', JaniceCreator.loadAnimData);
    $('saveAnimData').observe('click', JaniceCreator.saveAnimData);
    $('currentTime').observe('change', JaniceCreator.changeCurrentTime);
    $('animLength').observe('change', JaniceCreator.changeAnimLength);
    $('animObjects').observe('click', JaniceCreator.onClickAnimObjects);
    $('containerStack').observe('click', JaniceCreator.onClickContainerStack);
    $('createAnimObject').observe('click', JaniceCreator.onClickCreateAnimObject);

    // Fill object-types in select element:
    var objectTypesElem = $('objectTypes');
    objectTypesElem.insert(new Element('option', { 'value': 'container' }).update('container'));
    for (var blueprintName in Janice.BlueprintManager.blueprints) {
        objectTypesElem.insert(new Element('option', { 'value': blueprintName }).update(blueprintName));
    }

    // Create a new animation:
    JaniceCreator.newAnimation();
});


document.observe('dom:loaded', JaniceCreator.callOnDomReadyFunctions);


